/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * Cylinder.h
 *
 *  Created on: Mar 21, 2011
 *      Author: lziegler
 */

#ifndef CYLINDER_H_
#define CYLINDER_H_

#include <xmltio/xmltio.hpp>
#include "../../Type.h"
#include "../../units/UnitConverter.h"
#include "Point3D.h"
#include "Rotation3D.h"

namespace airobots {
/**
 *	A Cylinder.
 *
 * @author lziegler
 */
class Cylinder: public Type {
public:

	/**
	 * Default constructor. Position (0,0,0) Rotation (no rotation) Height (0)
	 * Radius (0).
	 */
	Cylinder();

	/**
	 * Constructor.
	 * @param position
	 *         Position of the cylinder (center).
	 * @param orientation
	 *         Orientation of the axis of the cylinder.
	 * @param height
	 *         Height of the cylinder.
	 * @param radius
	 *         Radius of the cylinder.
	 * @param lengthUnit
	 *         Unit of the height and radius values.
	 */
	Cylinder(const Point3D &position, const Rotation3D &orientation,
			double height, double radius, LengthUnit lengthUnit);

	/**
	 * Destructor.
	 */
	virtual ~Cylinder();

	static std::string getBaseTag();
	static std::string getTemplate();

	bool operator==(const Cylinder &other) const;

	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Cylinder &type);
	void sanitizeLocation(xmltio::Location &loc) const;

	/**
	 * Get position of the cylinder's center.
	 * @return
	 *         Position of the cylinder (center).
	 */
	Point3D getPosition() const;

	/**
	 * Get orientation of the cylinder's axis.
	 * @return
	 *         Orientation of the cylinder (axis).
	 */
	Rotation3D getOrientation() const;
	double getHeight(LengthUnit unit) const;
	double getRadius(LengthUnit unit) const;

	/**
	 * Set position of the cylinder's center.
	 * @param position
	 *         Position of the cylinder (center).
	 */
	void setPosition(const Point3D &position);
	/**
	 * Set orientation of the cylinder's axis.
	 * @param orientation
	 *         Orientation of the cylinder (axis).
	 */
	void setOrientation(const Rotation3D &orientation);
	void setHeight(double height, LengthUnit unit);
	void setRadius(double radius, LengthUnit unit);

private:
	Point3D position;
	Rotation3D orientation;
	double height;
	double radius;

	static const AngleUnit internalAngleUnit;
	static const LengthUnit internalLengthUnit;
};

}

#endif /* CYLINDER_H_ */
