package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This is a serializable cylinder type.
 * 
 * @author lziegler
 */
public class Cylinder extends Type {

	private static LengthUnit internalLengthUnit = LengthUnit.MILLIMETER;

	public static String POSITION_TAG = "POSITION";
	public static String RADIUS_TAG = "RADIUS";
	public static String HEIGHT_TAG = "HEIGHT";

	private Point3D position;
	private Rotation3D orientation;
	private double height;
	private double radius;

	/**
	 * Copy-constructor.
	 * 
	 * @param c
	 *            The instance to create a copy from.
	 */
	public Cylinder(Cylinder c) {

		this.position = new Point3D(c.getPosition().getX(internalLengthUnit), c
				.getPosition().getY(internalLengthUnit), c.getPosition().getZ(
				internalLengthUnit), internalLengthUnit);
		this.orientation = c.getOrientation();
		this.radius = c.getRadius(internalLengthUnit);
		this.height = c.getHeight(internalLengthUnit);
	}

	/**
	 * Constructor.
	 * 
	 * @param position
	 *            Center of the cylinder.
	 * @param rotationX
	 *            Rotation of the cylinder's axis around x-axis.
	 * @param rotationY
	 *            Rotation of the cylinder's axis around y-axis.
	 * @param rotationZ
	 *            Rotation of the cylinder's axis around z-axis.
	 * @param angleUnit
	 *            Unit of the given rotation angles.
	 * @param height
	 *            Height of the cylinder.
	 * @param radius
	 *            Radius of the cylinder.
	 * @param lengthUnit
	 *            Unit of the height and radius values.
	 */
	public Cylinder(Point3D position, Rotation3D direction, double height,
			double radius, LengthUnit lengthUnit) {
		this.position = position;
		this.orientation = direction;
		this.height = UnitConverter.convert(height, lengthUnit,
				internalLengthUnit);
		this.radius = UnitConverter.convert(radius, lengthUnit,
				internalLengthUnit);
	}

	/**
	 * Default constructor.
	 */
	public Cylinder() {
		this.position = new Point3D();
		this.orientation = new Rotation3D();
		this.height = 0;
		this.radius = 0;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "CYLINDER";
	}

	/**
	 * Constructs a {@link Cylinder} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link Cylinder} object containing all the information given
	 *         by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element objectElement, Cylinder cylinder)
			throws ParsingException {

		try {

			if (!objectElement.getLocalName().equals(getBaseTag())) {
				throw new ParsingException("Parsing Error: Base tag "
						+ "for polygon object must be '" + getBaseTag() + "'");
			}

			Element pos = objectElement.getFirstChildElement(POSITION_TAG);
			double px = Double.parseDouble(pos.getAttribute("x").getValue());
			double py = Double.parseDouble(pos.getAttribute("y").getValue());
			double pz = Double.parseDouble(pos.getAttribute("z").getValue());
			Element radius = objectElement.getFirstChildElement(RADIUS_TAG);
			double r = Double.parseDouble(radius.getAttribute("r").getValue());
			Element height = objectElement.getFirstChildElement(HEIGHT_TAG);
			double h = Double.parseDouble(height.getAttribute("h").getValue());

			Element rotElem = objectElement.getFirstChildElement(Rotation3D
					.getBaseTag());
			if (rotElem == null) {
				throw new ParsingException("Missing element "
						+ Rotation3D.getBaseTag());
			}
			Rotation3D orientation = new Rotation3D();
			Rotation3D.fromElement(rotElem, orientation);

			cylinder.setPosition(new Point3D(px, py, pz, internalLengthUnit));
			cylinder.setOrientation(orientation);
			cylinder.setHeight(h, internalLengthUnit);
			cylinder.setRadius(r, internalLengthUnit);

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present
			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new ParsingException("could not parse coordinate values");
		}
	}

	/**
	 * Serializes the {@link Cylinder} object into a given XOM {@link Element}.
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {

		Elements oldPositions = parent.getChildElements(POSITION_TAG);
		for (int i = 0; i < oldPositions.size(); i++) {
			parent.removeChild(oldPositions.get(i));
		}
		Element pos = new Element(POSITION_TAG);
		pos.addAttribute(new Attribute("x", String.valueOf(position
				.getX(internalLengthUnit))));
		pos.addAttribute(new Attribute("y", String.valueOf(position
				.getY(internalLengthUnit))));
		pos.addAttribute(new Attribute("z", String.valueOf(position
				.getZ(internalLengthUnit))));
		parent.appendChild(pos);

		Elements oldAxes = parent.getChildElements(Rotation3D.getBaseTag());
		for (int i = 0; i < oldAxes.size(); i++) {
			parent.removeChild(oldAxes.get(i));
		}

		Element dir = new Element(Rotation3D.getBaseTag());
		orientation.fillInto(dir);
		parent.appendChild(dir);

		Elements oldRadius = parent.getChildElements(RADIUS_TAG);
		for (int i = 0; i < oldRadius.size(); i++) {
			parent.removeChild(oldRadius.get(i));
		}
		Element radius = new Element(RADIUS_TAG);
		radius.addAttribute (new Attribute ("r", String.valueOf(getRadius(internalLengthUnit))));
		parent.appendChild(radius);

		Elements oldHeight = parent.getChildElements(HEIGHT_TAG);
		for (int i = 0; i < oldHeight.size(); i++) {
			parent.removeChild(oldHeight.get(i));
		}
		Element height = new Element(HEIGHT_TAG);
		height.addAttribute (new Attribute ("h", String.valueOf(getHeight(internalLengthUnit))));
		parent.appendChild(height);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String info = "[" + getBaseTag();
		info += " position=" + position;
		info += " orientation=" + orientation;
		info += " height=" + height;
		info += " radius=" + radius;
		info += "]";
		return info;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof Cylinder))
				return false;

			Cylinder other = (Cylinder) obj;

			return other.getPosition().equals(position)
					&& other.getOrientation().equals(getOrientation())
					&& other.getHeight(internalLengthUnit) == getHeight(internalLengthUnit)
					&& other.getRadius(internalLengthUnit) == getRadius(internalLengthUnit);
		} catch (Exception e) {
			return false;
		}
	}

	public Point3D getPosition() {
		return position;
	}

	public void setPosition(Point3D position) {
		this.position = position;
	}

	public Rotation3D getOrientation() {
		return orientation;
	}

	public void setOrientation(Rotation3D orientation) {
		this.orientation = orientation;
	}

	public double getRadius(LengthUnit unit) {
		return UnitConverter.convert(radius, internalLengthUnit, unit);
	}

	public void setRadius(double radius, LengthUnit unit) {
		this.radius = UnitConverter.convert(radius, unit, internalLengthUnit);
	}

	public double getHeight(LengthUnit unit) {
		return UnitConverter.convert(height, internalLengthUnit, unit);
	}

	public void setHeight(double height, LengthUnit unit) {
		this.height = UnitConverter.convert(height, unit, internalLengthUnit);
	}
}
