package de.unibi.airobots.btl.data.grasp;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.units.*;

public class KatanaPoseData extends Type {

	private double x = 0;
	private double y = 0;
	private double z = 0;
	private double phi = 0;
	private double psi = 0;
	private double theta = 0;
	private LengthUnit iLU = LengthUnit.MILLIMETER;
	private AngleUnit iAU = AngleUnit.RADIAN;

	/**
	 * Constructor.
	 */
	public KatanaPoseData() {
		super();
	}

	public static String getBaseTag() {
		return "POSITION";
	}

	/**
	 * Constructor.
	 */
	public KatanaPoseData(double x, double y, double z, double phi, double psi,
			double theta,AngleUnit aU, LengthUnit lU) {
		super();
		this.x = UnitConverter.convert(x, lU, iLU);
		this.y = UnitConverter.convert(y, lU, iLU);
		this.z = UnitConverter.convert(z, lU, iLU);
		this.psi = UnitConverter.convert(psi, aU, iAU);
		this.phi = UnitConverter.convert(phi, aU, iAU);
		this.theta = UnitConverter.convert(theta, aU, iAU);
	}

	/**
	 * Constructor.
	 */
	public KatanaPoseData(KatanaPoseData kpd) {
		super();
		this.x = kpd.x;
		this.y = kpd.y;
		this.z = kpd.z;
		this.psi = kpd.psi;
		this.phi = kpd.phi;
		this.theta = kpd.theta;
	}

	/**
	 * Returns the x-value of the endeffector position (mm)
	 * 
	 * @return x-value of the endeffector position
	 */
	public double getX(LengthUnit lU) {
		return UnitConverter.convert(x, iLU, lU);
	}

	/**
	 * Sets the x-value of the endeffector position (mm)
	 * 
	 * @param x
	 *            x-value of the endeffector
	 */
	public void setX(double x,LengthUnit lU) {
		this.x = UnitConverter.convert(x, lU, iLU);
	}

	/**
	 * Returns the y-value of the endeffector position (mm)
	 * 
	 * @return y-value of the endeffector position
	 */
	public double getY(LengthUnit lU) {
		return UnitConverter.convert(y, iLU, lU);
	}

	/**
	 * Sets the y-value of the endeffector position (mm)
	 * 
	 * @param y
	 *            y-value of the endeffector
	 */
	public void setY(double y,LengthUnit lU) {
		this.y = UnitConverter.convert(y, lU, iLU);
	}

	/**
	 * Returns the z-value of the endeffector position (mm)
	 * 
	 * @return z-value of the endeffector position
	 */
	public double getZ(LengthUnit lU) {
		return UnitConverter.convert(z, iLU, lU);
	}

	/**
	 * Sets the z-value of the endeffector position (mm)
	 * 
	 * @param z
	 *            z-value of the endeffector
	 */
	public void setZ(double z,LengthUnit lU) {
		this.z = UnitConverter.convert(z, lU, iLU);
	}

	/**
	 * Returns the phi-value of the endeffector position (mm)
	 * 
	 * @return phi-value of the endeffector position
	 */
	public double getPhi(AngleUnit aU) {
		return UnitConverter.convert(phi, iAU, aU);
	}

	/**
	 * Sets the phi-value of the endeffector position (rad)
	 * 
	 * @param phi
	 *            phi-value of the endeffector
	 */
	public void setPhi(double phi,AngleUnit aU) {
		this.phi = UnitConverter.convert(phi, aU, iAU);
	}

	/**
	 * Returns the psi-value of the endeffector position (mm)
	 * 
	 * @return psi-value of the endeffector position
	 */
	public double getPsi(AngleUnit aU) {
		return UnitConverter.convert(psi, iAU, aU);
	}

	/**
	 * Sets the psi-value of the endeffector position (rad)
	 * 
	 * @param psi
	 *            psi-value of the endeffector
	 */
	public void setPsi(double psi,AngleUnit aU) {
		this.psi = UnitConverter.convert(psi, aU, iAU);
	}

	/**
	 * Returns the theta-value of the endeffector position (rad)
	 * 
	 * @return theta-value of the endeffector position
	 */
	public double getTheta(AngleUnit aU) {
		return UnitConverter.convert(theta, iAU, aU);
	}

	/**
	 * Sets the theta-value of the endeffector position (rad)
	 * 
	 * @param theta
	 *            theta-value of the endeffector
	 */
	public void setTheta(double theta,AngleUnit aU) {
		this.theta = UnitConverter.convert(theta, aU, iAU);
	}

	public static void fromElement(Element element, KatanaPoseData type) {

		Type.fromElement(element, type);

		Element poseElement = element.getFirstChildElement("P");
		type.setX(ElementParser.getDoubleAttributeValue(poseElement, "x"),LengthUnit.MILLIMETER);
		type.setY(ElementParser.getDoubleAttributeValue(poseElement, "y"),LengthUnit.MILLIMETER);
		type.setZ(ElementParser.getDoubleAttributeValue(poseElement, "z"),LengthUnit.MILLIMETER);

		Element angleElement = element.getFirstChildElement("ANGLE");
		type.setPhi(ElementParser.getDoubleAttributeValue(angleElement, "phi"),AngleUnit.RADIAN);
		type.setPsi(ElementParser.getDoubleAttributeValue(angleElement, "psi"),AngleUnit.RADIAN);
		type.setTheta(ElementParser.getDoubleAttributeValue(angleElement,
				"theta"),AngleUnit.RADIAN);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element poseElement = new Element("P");
		parent.appendChild(poseElement);
		poseElement.addAttribute(new Attribute("x", String.valueOf(getX(LengthUnit.MILLIMETER))));
		poseElement.addAttribute(new Attribute("y", String.valueOf(getY(LengthUnit.MILLIMETER))));
		poseElement.addAttribute(new Attribute("z", String.valueOf(getZ(LengthUnit.MILLIMETER))));

		Element angleElement = new Element("ANGLE");
		parent.appendChild(angleElement);
		angleElement
				.addAttribute(new Attribute("phi", String.valueOf(getPhi(AngleUnit.RADIAN))));
		angleElement
				.addAttribute(new Attribute("psi", String.valueOf(getPsi(AngleUnit.RADIAN))));
		angleElement.addAttribute(new Attribute("theta", String
				.valueOf(getTheta(AngleUnit.RADIAN))));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "KatanaPoseData [x = " + x + ", y = " + y + ", z = " + z
				+ ", phi = " + phi + ", psi = " + psi + ", theta = " + theta
				+ "]";
	}

}
