package de.unibi.airobots.btl.data.navigation;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.airobots.btl.Type;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 *
 * @author pdressel
 */
public class ROSBaseLocalPlannerConfiguration extends ROSDynamicConfiguration{
 
    private static final String BASELOCALPLANNER_CONFIGURATION_TAG = "BASELOCALPLANNERCONFIGURATION";
    
    public static final String ACC_LIM_X_ATTRIBUTE = "acc_lim_x";    
    public static final String ACC_LIM_Y_ATTRIBUTE = "acc_lim_y";
    public static final String ACC_LIM_THETA_ATTRIBUTE = "acc_lim_theta";
    public static final String MAX_VEL_X_ATTRIBUTE = "max_vel_x";
    public static final String MIN_VEL_X_ATTRIBUTE = "min_vel_x";
    public static final String MAX_VEL_THETA_ATTRIBUTE = "max_vel_theta";
    public static final String MIN_VEL_THETA_ATTRIBUTE = "min_vel_theta";
    public static final String MIN_IN_PLACE_VEL_THETA_ATTRIBUTE = "min_in_place_vel_theta";
    public static final String SIM_TIME_ATTRIBUTE = "sim_time";
    public static final String SIM_GRANULARITY_ATTRIBUTE = "sim_granularity";
    public static final String PDIST_SCALE_ATTRIBUTE = "pdist_scale";
    public static final String GDIST_SCALE_ATTRIBUTE = "gdist_scale";
    public static final String OCCDIST_SCALE_ATTRIBUTE = "occdist_scale";
    public static final String OSCILLATION_RESET_DIST_ATTRIBUTE = "oscillation_reset_dist";
    public static final String ESCAPE_RESET_DIST_ATTRIBUTE = "escape_reset_dist";
    public static final String ESCAPE_RESET_THETA_ATTRIBUTE = "escape_reset_theta";
    public static final String VX_SAMPLES_ATTRIBUTE = "vx_samples";
    public static final String VTHETA_SAMPLES_ATTRIBUTE = "vtheta_samples";
    public static final String HEADING_LOOKAHEAD_ATTRIBUTE = "heading_lookahead";
    public static final String HOLONOMIC_ROBOT_ATTRIBUTE = "holonomic_robot";
    public static final String ESCAPE_VEL_ATTRIBUTE = "escape_vel";
    public static final String DWA_ATTRIBUTE = "dwa";
    public static final String HEADING_SCORING_ATTRIBUTE = "heading_scoring";
    public static final String HEADING_SCORING_TIMESTAMP_ATTRIBUTE = "heading_scoring_timestep";
    public static final String SIMPLE_ATTRACTOR_ATTRIBUTE = "simple_attractor";
    public static final String ANGULAR_SIM_GRANULARITY_ATTRIBUTE = "angular_sim_granularity";
    public static final String Y_VELS_ATTRIBUTE = "y_vels";
    public static final String RESTORE_DEFAULTS_ATTRIBUTE = "restore_defaults";
    
    /**
     * Explanation: The x acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_x'.<br>
     * Type and default: double, 2.5.
     */
    private double acc_lim_x;
    
    /**
     * Explanation: The y acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_y'.<br>
     * Type and default: double, 2.5.
     */
    private double acc_lim_y;

    /**
     * Explanation: The rotational acceleration limit of the robot in 
     * radians/sec^2 .<br>
     * name: '<node_name>/acc_lim_th'.<br>
     * Type and default: double, 3.2.
     */
    private double acc_lim_theta;

    /**
     * Explanation: The maximum forward velocity allowed for the base in 
     * meters/sec.<br>
     * name: '<node_name>/max_vel_x'.<br>
     * Type and default: double, 0.5.
     */
    private double max_vel_x;

    /**
     * Explanation: The minimum forward velocity allowed for the base in 
     * meters/sec. It is useful to specify this to guarantee that velocity 
     * commands sent to a mobile base are high enough to allow the base to 
     * overcome friction.<br>
     * name: '<node_name>/min_vel_x'.<br>
     * Type and default: double, 0.1.
     */
    private double min_vel_x;

    /**
     * Explanation: The maximum rotational velocity allowed for the base in 
     * radians/sec.<br>
     * name: '<node_name>/max_rotational_vel'.<br>
     * Type and default: double, 1.0.
     */
    private double max_vel_theta;

    /**
     * Explanation: The minimum rotational velocity allowed for the base in 
     * radians/sec.<br>
     * name: '<node_name>/min_rotational_vel'.<br>
     * Type and default: double, 0.0.
     */
    private double min_vel_theta;

    /**
     * Explanation: The minimum rotational velocity allowed for the base while 
     * performing in-place rotations in radians/sec.<br>
     * name: '<node_name>/min_in_place_rotational_vel'.<br>
     * Type and default: double, 0.4.
     */    
    private double min_in_place_vel_theta;

    /**
     * Explanation: The amount of time to forward-simulate trajectories in seconds.<br>
     * name: '<node_name>/sim_time'.<br>
     * Type and default: double, 1.0.
     */    
    private double sim_time;

    /**
     * Explanation: The step size, in meters, to take between points on a given 
     * trajectory.<br>
     * name: '<node_name>/sim_granularity'.<br>
     * Type and default: double, 0.025.
     */    
    private double sim_granularity;
    
    /**
     * Explanation: The weighting for how much the controller should stay close 
     * to the path it was given.<br>
     * name: '<node_name>/pdist_scale'.<br>
     * Type and default: double, 0.6.
     */    
    private double pdist_scale;

    /**
     * Explanation: The weighting for how much the controller should attempt to 
     * reach its local goal, also controls speed.<br>
     * name: '<node_name>/gdist_scale'.<br>
     * Type and default: double, 0.8.
     */    
    private double gdist_scale;

    /**
     * Explanation: The weighting for how much the controller should attempt to 
     * avoid obstacles.<br>
     * name: '<node_name>/occdist_scale'.<br>
     * Type and default: double, 0.01.
     */    
    private double occdist_scale;

    /**
     * Explanation: How far the robot must travel in meters before oscillation 
     * flags are reset.<br>
     * name: '<node_name>/oscillation_reset_dist'.<br>
     * Type and default: double, 0.05.
     */    
    private double oscillation_reset_dist;

    /**
     * Explanation: The distance the robot must travel before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_dist'.<br>
     * Type and default: double, 0.1.
     */
    private double escape_reset_dist;

    /**
     * Explanation: The distance the robot must rotate before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_theta'.<br>
     * Type and default: double, 1.5708 (PI/2).
     */    
    private double escape_reset_theta;

    /**
     * Explanation: The number of samples to use when exploring the x velocity 
     * space.<br>
     * name: '<node_name>/vx_samples'.<br>
     * Type and default: integer, 3.
     */    
    private int vx_samples;

    /**
     * Explanation: The number of samples to use when exploring the theta velocity space .<br>
     * name: '<node_name>/vtheta_samples'.<br>
     * Type and default: integer, 20.
     */    
    private int vtheta_samples;

    /**
     * Explanation: How far to look ahead in meters when scoring different 
     * in-place-rotation trajectories.<br>
     * name: '<node_name>/heading_lookahead'.<br>
     * Type and default: double, 0.325.
     */    
    private double heading_lookahead;
    
    /**
     * Explanation: Determines whether velocity commands are generated for a 
     * holonomic or non-holonomic robot. For holonomic robots, strafing velocity 
     * commands may be issued to the base. For non-holonomic robots, no strafing 
     * velocity commands will be issued.<br>
     * name: '<node_name>/holonomic_robot'.<br>
     * Type and default: bool, true.
     */    
    private boolean holonomic_robot;

    /**
     * Explanation: Speed used for driving during escapes in meters/sec. Note 
     * that it must be negative in order for the robot to actually reverse. A 
     * positive speed will cause the robot to move forward while attempting to 
     * escape.<br>
     * name: '<node_name>/escape_vel'.<br>
     * Type and default: double, -0.1.
     */    
    private double escape_vel;

    /**
     * Explanation: Whether to use the Dynamic Window Approach (DWA)_ or whether
     * to use Trajectory Rollout (NOTE: In our experience DWA worked as well as 
     * Trajectory Rollout and is computationally less expensive. It is possible 
     * that robots with extremely poor acceleration limits could gain from 
     * running Trajectory Rollout).<br>
     * name: '<node_name>/dwa'.<br>
     * Type and default: bool, true.
     */    
    private boolean dwa;
    
    /**
     * Explanation: Whether to score based on the robot's heading to the path 
     * or its distance from the path.<br>
     * name: '<node_name>/heading_scoring'.<br>
     * Type and default: bool, false.
     */    
    private boolean heading_scoring;
    
    /**
     * Explanation: How far to look ahead in time in seconds along the simulated
     * trajectory when using heading scoring.<br>
     * name: '<node_name>/heading_scoring_timestep'.<br>
     * Type and default: double, 0.8.
     */    
    private double heading_scoring_timestep;

    /**
     * Explanation: Set this to true to allow simple attraction to a goal point 
     * instead of intelligent cost propagation.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: bool, false.
     */    
    private boolean simple_attractor;

    /**
     * Explanation: The step size, in radians, to take between angular samples 
     * on a given trajectory.<br>
     * name: '<node_name>/angular_sim_granularity'.<br>
     * Type and default: double, 0.025.
     */    
    private double angular_sim_granularity;
    
    /**
     * Explanation: The strafing velocities that a holonomic robot will consider in meters/sec.<br>
     * name: '<node_name>/y_vels'.<br>
     * Type and default: list, '[-0.3, -0.1, 0.1, 0.3]'.
     */    
    private String y_vels;

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     */  
    private boolean restore_defaults;    
    
    
    public ROSBaseLocalPlannerConfiguration()
    {
        // Fill all the properties with values from the current configuration as
        // is on the ROS server.
        // DO THIS IN BonSAI  
    }

    /**
     * Explanation: The x acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_x'.<br>
     * Type and default: double, 2.5.
     * @return the acc_lim_x
     */
    public double getAccLimX() {
        return acc_lim_x;
    }

    /**
     * Explanation: The x acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_x'.<br>
     * Type and default: double, 2.5.
     * @param acc_lim_x the acc_lim_x to set
     */
    public void setAccLimX(double acc_lim_x) {
        this.acc_lim_x = acc_lim_x;
    }

    /**
     * Explanation: The y acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_y'.<br>
     * Type and default: double, 2.5.
     * @return the acc_lim_y
     */
    public double getAccLimY() {
        return acc_lim_y;
    }

    /**
     * Explanation: The y acceleration limit of the robot in meters/sec^2.<br>
     * name: '<node_name>/acc_lim_y'.<br>
     * Type and default: double, 2.5.
     * @param acc_lim_y the acc_lim_y to set
     */
    public void setAccLimY(double acc_lim_y) {
        this.acc_lim_y = acc_lim_y;
    }

    /**
     * Explanation: The rotational acceleration limit of the robot in
     * radians/sec^2 .<br>
     * name: '<node_name>/acc_lim_th'.<br>
     * Type and default: double, 3.2.
     * @return the acc_lim_theta
     */
    public double getAccLimTheta() {
        return acc_lim_theta;
    }

    /**
     * Explanation: The rotational acceleration limit of the robot in
     * radians/sec^2 .<br>
     * name: '<node_name>/acc_lim_th'.<br>
     * Type and default: double, 3.2.
     * @param acc_lim_theta the acc_lim_theta to set
     */
    public void setAccLimTheta(double acc_lim_theta) {
        this.acc_lim_theta = acc_lim_theta;
    }

    /**
     * Explanation: The maximum forward velocity allowed for the base in
     * meters/sec.<br>
     * name: '<node_name>/max_vel_x'.<br>
     * Type and default: double, 0.5.
     * @return the max_vel_x
     */
    public double getMaxVelX() {
        return max_vel_x;
    }

    /**
     * Explanation: The maximum forward velocity allowed for the base in
     * meters/sec.<br>
     * name: '<node_name>/max_vel_x'.<br>
     * Type and default: double, 0.5.
     * @param max_vel_x the max_vel_x to set
     */
    public void setMaxVelX(double max_vel_x) {
        this.max_vel_x = max_vel_x;
    }

    /**
     * Explanation: The minimum forward velocity allowed for the base in
     * meters/sec. It is useful to specify this to guarantee that velocity
     * commands sent to a mobile base are high enough to allow the base to
     * overcome friction.<br>
     * name: '<node_name>/min_vel_x'.<br>
     * Type and default: double, 0.1.
     * @return the min_vel_x
     */
    public double getMinVelX() {
        return min_vel_x;
    }

    /**
     * Explanation: The minimum forward velocity allowed for the base in
     * meters/sec. It is useful to specify this to guarantee that velocity
     * commands sent to a mobile base are high enough to allow the base to
     * overcome friction.<br>
     * name: '<node_name>/min_vel_x'.<br>
     * Type and default: double, 0.1.
     * @param min_vel_x the min_vel_x to set
     */
    public void setMinVelX(double min_vel_x) {
        this.min_vel_x = min_vel_x;
    }

    /**
     * Explanation: The maximum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/max_rotational_vel'.<br>
     * Type and default: double, 1.0.
     * @return the max_vel_theta
     */
    public double getMaxVelTheta() {
        return max_vel_theta;
    }

    /**
     * Explanation: The maximum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/max_rotational_vel'.<br>
     * Type and default: double, 1.0.
     * @param max_vel_theta the max_vel_theta to set
     */
    public void setMaxVelTheta(double max_vel_theta) {
        this.max_vel_theta = max_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/min_rotational_vel'.<br>
     * Type and default: double, 0.0.
     * @return the min_vel_theta
     */
    public double getMinVelTheta() {
        return min_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base in
     * radians/sec.<br>
     * name: '<node_name>/min_rotational_vel'.<br>
     * Type and default: double, 0.0.
     * @param min_vel_theta the min_vel_theta to set
     */
    public void setMinVelTheta(double min_vel_theta) {
        this.min_vel_theta = min_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base while
     * performing in-place rotations in radians/sec.<br>
     * name: '<node_name>/min_in_place_rotational_vel'.<br>
     * Type and default: double, 0.4.
     * @return the min_in_place_vel_theta
     */
    public double getMinInPlaceVelTheta() {
        return min_in_place_vel_theta;
    }

    /**
     * Explanation: The minimum rotational velocity allowed for the base while
     * performing in-place rotations in radians/sec.<br>
     * name: '<node_name>/min_in_place_rotational_vel'.<br>
     * Type and default: double, 0.4.
     * @param min_in_place_vel_theta the min_in_place_vel_theta to set
     */
    public void setMinInPlaceVelTheta(double min_in_place_vel_theta) {
        this.min_in_place_vel_theta = min_in_place_vel_theta;
    }

    /**
     * Explanation: The amount of time to forward-simulate trajectories in seconds.<br>
     * name: '<node_name>/sim_time'.<br>
     * Type and default: double, 1.0.
     * @return the sim_time
     */
    public double getSimTime() {
        return sim_time;
    }

    /**
     * Explanation: The amount of time to forward-simulate trajectories in seconds.<br>
     * name: '<node_name>/sim_time'.<br>
     * Type and default: double, 1.0.
     * @param sim_time the sim_time to set
     */
    public void setSimTime(double sim_time) {
        this.sim_time = sim_time;
    }

    /**
     * Explanation: The step size, in meters, to take between points on a given
     * trajectory.<br>
     * name: '<node_name>/sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @return the sim_granularity
     */
    public double getSimGranularity() {
        return sim_granularity;
    }

    /**
     * Explanation: The step size, in meters, to take between points on a given
     * trajectory.<br>
     * name: '<node_name>/sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @param sim_granularity the sim_granularity to set
     */
    public void setSimGranularity(double sim_granularity) {
        this.sim_granularity = sim_granularity;
    }

    /**
     * Explanation: The weighting for how much the controller should stay close
     * to the path it was given.<br>
     * name: '<node_name>/pdist_scale'.<br>
     * Type and default: double, 0.6.
     * @return the pdist_scale
     */
    public double getPdistScale() {
        return pdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should stay close
     * to the path it was given.<br>
     * name: '<node_name>/pdist_scale'.<br>
     * Type and default: double, 0.6.
     * @param pdist_scale the pdist_scale to set
     */
    public void setPdistScale(double pdist_scale) {
        this.pdist_scale = pdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * reach its local goal, also controls speed.<br>
     * name: '<node_name>/gdist_scale'.<br>
     * Type and default: double, 0.8.
     * @return the gdist_scale
     */
    public double getGdistScale() {
        return gdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * reach its local goal, also controls speed.<br>
     * name: '<node_name>/gdist_scale'.<br>
     * Type and default: double, 0.8.
     * @param gdist_scale the gdist_scale to set
     */
    public void setGdistScale(double gdist_scale) {
        this.gdist_scale = gdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * avoid obstacles.<br>
     * name: '<node_name>/occdist_scale'.<br>
     * Type and default: double, 0.01.
     * @return the occdist_scale
     */
    public double getOccdistScale() {
        return occdist_scale;
    }

    /**
     * Explanation: The weighting for how much the controller should attempt to
     * avoid obstacles.<br>
     * name: '<node_name>/occdist_scale'.<br>
     * Type and default: double, 0.01.
     * @param occdist_scale the occdist_scale to set
     */
    public void setOccdistScale(double occdist_scale) {
        this.occdist_scale = occdist_scale;
    }

    /**
     * Explanation: How far the robot must travel in meters before oscillation
     * flags are reset.<br>
     * name: '<node_name>/oscillation_reset_dist'.<br>
     * Type and default: double, 0.05.
     * @return the oscillation_reset_dist
     */
    public double getOscillationResetDist() {
        return oscillation_reset_dist;
    }

    /**
     * Explanation: How far the robot must travel in meters before oscillation
     * flags are reset.<br>
     * name: '<node_name>/oscillation_reset_dist'.<br>
     * Type and default: double, 0.05.
     * @param oscillation_reset_dist the oscillation_reset_dist to set
     */
    public void setOscillationResetDist(double oscillation_reset_dist) {
        this.oscillation_reset_dist = oscillation_reset_dist;
    }

    /**
     * Explanation: The distance the robot must travel before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_dist'.<br>
     * Type and default: double, 0.1.
     * @return the escape_reset_dist
     */
    public double getEscapeResetDist() {
        return escape_reset_dist;
    }

    /**
     * Explanation: The distance the robot must travel before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_dist'.<br>
     * Type and default: double, 0.1.
     * @param escape_reset_dist the escape_reset_dist to set
     */
    public void setEscapeResetDist(double escape_reset_dist) {
        this.escape_reset_dist = escape_reset_dist;
    }

    /**
     * Explanation: The distance the robot must rotate before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_theta'.<br>
     * Type and default: double, 1.5708 (PI/2).
     * @return the escape_reset_theta
     */
    public double getEscapeResetTheta() {
        return escape_reset_theta;
    }

    /**
     * Explanation: The distance the robot must rotate before it can exit escape
     * mode.<br>
     * name: '<node_name>/escape_reset_theta'.<br>
     * Type and default: double, 1.5708 (PI/2).
     * @param escape_reset_theta the escape_reset_theta to set
     */
    public void setEscapeResetTheta(double escape_reset_theta) {
        this.escape_reset_theta = escape_reset_theta;
    }

    /**
     * Explanation: The number of samples to use when exploring the x velocity
     * space.<br>
     * name: '<node_name>/vx_samples'.<br>
     * Type and default: integer, 3.
     * @return the vx_samples
     */
    public int getVxSamples() {
        return vx_samples;
    }

    /**
     * Explanation: The number of samples to use when exploring the x velocity
     * space.<br>
     * name: '<node_name>/vx_samples'.<br>
     * Type and default: integer, 3.
     * @param vx_samples the vx_samples to set
     */
    public void setVxSamples(int vx_samples) {
        this.vx_samples = vx_samples;
    }

    /**
     * Explanation: The number of samples to use when exploring the theta velocity space .<br>
     * name: '<node_name>/vtheta_samples'.<br>
     * Type and default: integer, 20.
     * @return the vtheta_samples
     */
    public int getVthetaSamples() {
        return vtheta_samples;
    }

    /**
     * Explanation: The number of samples to use when exploring the theta velocity space .<br>
     * name: '<node_name>/vtheta_samples'.<br>
     * Type and default: integer, 20.
     * @param vtheta_samples the vtheta_samples to set
     */
    public void setVthetaSamples(int vtheta_samples) {
        this.vtheta_samples = vtheta_samples;
    }

    /**
     * Explanation: How far to look ahead in meters when scoring different
     * in-place-rotation trajectories.<br>
     * name: '<node_name>/heading_lookahead'.<br>
     * Type and default: double, 0.325.
     * @return the heading_lookahead
     */
    public double getHeadingLookahead() {
        return heading_lookahead;
    }

    /**
     * Explanation: How far to look ahead in meters when scoring different
     * in-place-rotation trajectories.<br>
     * name: '<node_name>/heading_lookahead'.<br>
     * Type and default: double, 0.325.
     * @param heading_lookahead the heading_lookahead to set
     */
    public void setHeadingLookahead(double heading_lookahead) {
        this.heading_lookahead = heading_lookahead;
    }

    /**
     * Explanation: Determines whether velocity commands are generated for a
     * holonomic or non-holonomic robot. For holonomic robots, strafing velocity
     * commands may be issued to the base. For non-holonomic robots, no strafing
     * velocity commands will be issued.<br>
     * name: '<node_name>/holonomic_robot'.<br>
     * Type and default: bool, true.
     * @return the holonomic_robot
     */
    public boolean isHolonomicRobot() {
        return holonomic_robot;
    }

    /**
     * Explanation: Determines whether velocity commands are generated for a
     * holonomic or non-holonomic robot. For holonomic robots, strafing velocity
     * commands may be issued to the base. For non-holonomic robots, no strafing
     * velocity commands will be issued.<br>
     * name: '<node_name>/holonomic_robot'.<br>
     * Type and default: bool, true.
     * @param holonomic_robot the holonomic_robot to set
     */
    public void setHolonomicRobot(boolean holonomic_robot) {
        this.holonomic_robot = holonomic_robot;
    }

    /**
     * Explanation: Speed used for driving during escapes in meters/sec. Note
     * that it must be negative in order for the robot to actually reverse. A
     * positive speed will cause the robot to move forward while attempting to
     * escape.<br>
     * name: '<node_name>/escape_vel'.<br>
     * Type and default: double, -0.1.
     * @return the escape_vel
     */
    public double getEscapeVel() {
        return escape_vel;
    }

    /**
     * Explanation: Speed used for driving during escapes in meters/sec. Note
     * that it must be negative in order for the robot to actually reverse. A
     * positive speed will cause the robot to move forward while attempting to
     * escape.<br>
     * name: '<node_name>/escape_vel'.<br>
     * Type and default: double, -0.1.
     * @param escape_vel the escape_vel to set
     */
    public void setEscapeVel(double escape_vel) {
        this.escape_vel = escape_vel;
    }

    /**
     * Explanation: Whether to use the Dynamic Window Approach (DWA)_ or whether
     * to use Trajectory Rollout (NOTE: In our experience DWA worked as well as
     * Trajectory Rollout and is computationally less expensive. It is possible
     * that robots with extremely poor acceleration limits could gain from
     * running Trajectory Rollout).<br>
     * name: '<node_name>/dwa'.<br>
     * Type and default: bool, true.
     * @return the dwa
     */
    public boolean isDwa() {
        return dwa;
    }

    /**
     * Explanation: Whether to use the Dynamic Window Approach (DWA)_ or whether
     * to use Trajectory Rollout (NOTE: In our experience DWA worked as well as
     * Trajectory Rollout and is computationally less expensive. It is possible
     * that robots with extremely poor acceleration limits could gain from
     * running Trajectory Rollout).<br>
     * name: '<node_name>/dwa'.<br>
     * Type and default: bool, true.
     * @param dwa the dwa to set
     */
    public void setDwa(boolean dwa) {
        this.dwa = dwa;
    }

    /**
     * Explanation: Whether to score based on the robot's heading to the path
     * or its distance from the path.<br>
     * name: '<node_name>/heading_scoring'.<br>
     * Type and default: bool, false.
     * @return the heading_scoring
     */
    public boolean isHeadingScoring() {
        return heading_scoring;
    }

    /**
     * Explanation: Whether to score based on the robot's heading to the path
     * or its distance from the path.<br>
     * name: '<node_name>/heading_scoring'.<br>
     * Type and default: bool, false.
     * @param heading_scoring the heading_scoring to set
     */
    public void setHeadingScoring(boolean heading_scoring) {
        this.heading_scoring = heading_scoring;
    }

    /**
     * Explanation: How far to look ahead in time in seconds along the simulated
     * trajectory when using heading scoring.<br>
     * name: '<node_name>/heading_scoring_timestep'.<br>
     * Type and default: double, 0.8.
     * @return the heading_scoring_timestep
     */
    public double getHeadingScoringTimestep() {
        return heading_scoring_timestep;
    }

    /**
     * Explanation: How far to look ahead in time in seconds along the simulated
     * trajectory when using heading scoring.<br>
     * name: '<node_name>/heading_scoring_timestep'.<br>
     * Type and default: double, 0.8.
     * @param heading_scoring_timestep the heading_scoring_timestep to set
     */
    public void setHeadingScoringTimestep(double heading_scoring_timestep) {
        this.heading_scoring_timestep = heading_scoring_timestep;
    }

    /**
     * Explanation: Set this to true to allow simple attraction to a goal point
     * instead of intelligent cost propagation.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: bool, false.
     * @return the simple_attractor
     */
    public boolean isSimpleAttractor() {
        return simple_attractor;
    }

    /**
     * Explanation: Set this to true to allow simple attraction to a goal point
     * instead of intelligent cost propagation.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: bool, false.
     * @param simple_attractor the simple_attractor to set
     */
    public void setSimpleAttractor(boolean simple_attractor) {
        this.simple_attractor = simple_attractor;
    }

    /**
     * Explanation: The step size, in radians, to take between angular samples
     * on a given trajectory.<br>
     * name: '<node_name>/angular_sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @return the angular_sim_granularity
     */
    public double getAngularSimGranularity() {
        return angular_sim_granularity;
    }

    /**
     * Explanation: The step size, in radians, to take between angular samples
     * on a given trajectory.<br>
     * name: '<node_name>/angular_sim_granularity'.<br>
     * Type and default: double, 0.025.
     * @param angular_sim_granularity the angular_sim_granularity to set
     */
    public void setAngularSimGranularity(double angular_sim_granularity) {
        this.angular_sim_granularity = angular_sim_granularity;
    }

    /**
     * Explanation: The strafing velocities that a holonomic robot will consider in meters/sec.<br>
     * name: '<node_name>/y_vels'.<br>
     * Type and default: list, '[-0.3, -0.1, 0.1, 0.3]'.
     * @return the y_vels
     */
    public String getYVels() {
        return y_vels;
    }

    /**
     * Explanation: The strafing velocities that a holonomic robot will consider in meters/sec.<br>
     * name: '<node_name>/y_vels'.<br>
     * Type and default: list, '[-0.3, -0.1, 0.1, 0.3]'.
     * @param y_vels the y_vels to set
     */
    public void setYVels(String y_vels) {
        this.y_vels = y_vels;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @return the restore_defaults
     */
    public boolean isRestoreDefaults() {
        return restore_defaults;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @param restore_defaults the restore_defaults to set
     */
    public void setRestoreDefaults(boolean restore_defaults) {
        this.restore_defaults = restore_defaults;
    }
    
    /**
     * Fills an {@link ROSMoveBaseConfiguration} object from a given XOM {@link Element}
     *
     * @param objectElement The XOM {@link Element} to fill an object from.
     * @param type The {@link ROSCostmapConfiguration} object to fill with all
     * the information given by the {@link Element} object.
     */
    public static void fromElement(Element objectElement, ROSBaseLocalPlannerConfiguration type)
            throws ParsingException {

        Type.fromElement(objectElement, type);

        try {

            Nodes positionNodes = objectElement.query(BASELOCALPLANNER_CONFIGURATION_TAG);
            if (positionNodes.size() < 1
                    && !(positionNodes.get(0) instanceof Element)) {
                throw new ParsingException(
                        "Unable to find element with name " + BASELOCALPLANNER_CONFIGURATION_TAG + ".");
            }

            Element temp = (Element) positionNodes.get(0);

            type.acc_lim_theta = Double.parseDouble(temp.getAttribute(ACC_LIM_THETA_ATTRIBUTE).getValue());
            type.acc_lim_x = Double.parseDouble(temp.getAttribute(ACC_LIM_X_ATTRIBUTE).getValue());
            type.acc_lim_y = Double.parseDouble(temp.getAttribute(ACC_LIM_Y_ATTRIBUTE).getValue());
            type.angular_sim_granularity = Double.parseDouble(temp.getAttribute(ANGULAR_SIM_GRANULARITY_ATTRIBUTE).getValue());
            type.dwa = Boolean.parseBoolean(temp.getAttribute(DWA_ATTRIBUTE).getValue());
            type.escape_reset_dist = Double.parseDouble(temp.getAttribute(ESCAPE_RESET_DIST_ATTRIBUTE).getValue());
            type.escape_reset_theta = Double.parseDouble(temp.getAttribute(ESCAPE_RESET_THETA_ATTRIBUTE).getValue());
            type.escape_vel = Double.parseDouble(temp.getAttribute(ESCAPE_VEL_ATTRIBUTE).getValue());
            type.gdist_scale = Double.parseDouble(temp.getAttribute(GDIST_SCALE_ATTRIBUTE).getValue());
            type.heading_lookahead = Double.parseDouble(temp.getAttribute(HEADING_LOOKAHEAD_ATTRIBUTE).getValue());            
            type.heading_scoring = Boolean.parseBoolean(temp.getAttribute(HEADING_SCORING_ATTRIBUTE).getValue());
            type.heading_scoring_timestep = Double.parseDouble(temp.getAttribute(HEADING_SCORING_TIMESTAMP_ATTRIBUTE).getValue());
            type.holonomic_robot = Boolean.parseBoolean(temp.getAttribute(HOLONOMIC_ROBOT_ATTRIBUTE).getValue());
            type.max_vel_theta = Double.parseDouble(temp.getAttribute(MAX_VEL_THETA_ATTRIBUTE).getValue());
            type.max_vel_x = Double.parseDouble(temp.getAttribute(MAX_VEL_X_ATTRIBUTE).getValue());
            type.min_in_place_vel_theta = Double.parseDouble(temp.getAttribute(MIN_IN_PLACE_VEL_THETA_ATTRIBUTE).getValue());
            type.min_vel_theta = Double.parseDouble(temp.getAttribute(MIN_VEL_THETA_ATTRIBUTE).getValue());
            type.min_vel_x = Double.parseDouble(temp.getAttribute(MIN_VEL_X_ATTRIBUTE).getValue());
            type.occdist_scale = Double.parseDouble(temp.getAttribute(OCCDIST_SCALE_ATTRIBUTE).getValue());
            type.oscillation_reset_dist = Double.parseDouble(temp.getAttribute(OSCILLATION_RESET_DIST_ATTRIBUTE).getValue());
            type.pdist_scale = Double.parseDouble(temp.getAttribute(PDIST_SCALE_ATTRIBUTE).getValue());
            type.restore_defaults = Boolean.parseBoolean(temp.getAttribute(RESTORE_DEFAULTS_ATTRIBUTE).getValue());
            type.sim_granularity = Double.parseDouble(temp.getAttribute(SIM_GRANULARITY_ATTRIBUTE).getValue());
            type.sim_time = Double.parseDouble(temp.getAttribute(SIM_TIME_ATTRIBUTE).getValue());
            type.simple_attractor = Boolean.parseBoolean(temp.getAttribute(SIMPLE_ATTRACTOR_ATTRIBUTE).getValue());
            type.vtheta_samples = Integer.parseInt(temp.getAttribute(VTHETA_SAMPLES_ATTRIBUTE).getValue());
            type.vx_samples = Integer.parseInt(temp.getAttribute(VX_SAMPLES_ATTRIBUTE).getValue());
            type.y_vels = temp.getAttribute(Y_VELS_ATTRIBUTE).getValue();
            
        } catch (NullPointerException ex) {
            // this happens when an element or attribute that is required is
            // not present
            throw new IllegalArgumentException("Missing element or attribute "
                    + "in document.", ex);
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("Can not parse number", e);
        }
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void fillInto(Element parent) {

        sanitizeElement(parent);
        super.fillInto(parent);

        Element configuration = new Element(BASELOCALPLANNER_CONFIGURATION_TAG);

        configuration.addAttribute(new Attribute(ACC_LIM_THETA_ATTRIBUTE, String.valueOf(this.acc_lim_theta)));        
        configuration.addAttribute(new Attribute(ACC_LIM_X_ATTRIBUTE, String.valueOf(this.acc_lim_x)));
        configuration.addAttribute(new Attribute(ACC_LIM_Y_ATTRIBUTE, String.valueOf(this.acc_lim_y)));
        configuration.addAttribute(new Attribute(ANGULAR_SIM_GRANULARITY_ATTRIBUTE, String.valueOf(this.angular_sim_granularity)));
        configuration.addAttribute(new Attribute(DWA_ATTRIBUTE, String.valueOf(this.dwa)));
        configuration.addAttribute(new Attribute(ESCAPE_RESET_DIST_ATTRIBUTE, String.valueOf(this.escape_reset_dist)));
        configuration.addAttribute(new Attribute(ESCAPE_RESET_THETA_ATTRIBUTE, String.valueOf(this.escape_reset_theta)));
        configuration.addAttribute(new Attribute(ESCAPE_VEL_ATTRIBUTE, String.valueOf(this.escape_vel)));
        configuration.addAttribute(new Attribute(GDIST_SCALE_ATTRIBUTE, String.valueOf(this.gdist_scale)));
        configuration.addAttribute(new Attribute(HEADING_LOOKAHEAD_ATTRIBUTE, String.valueOf(this.heading_lookahead)));
        configuration.addAttribute(new Attribute(HEADING_SCORING_ATTRIBUTE, String.valueOf(this.heading_scoring)));
        configuration.addAttribute(new Attribute(HEADING_SCORING_TIMESTAMP_ATTRIBUTE, String.valueOf(this.heading_scoring_timestep)));
        configuration.addAttribute(new Attribute(HOLONOMIC_ROBOT_ATTRIBUTE, String.valueOf(this.holonomic_robot)));
        configuration.addAttribute(new Attribute(MAX_VEL_THETA_ATTRIBUTE, String.valueOf(this.max_vel_theta)));
        configuration.addAttribute(new Attribute(MAX_VEL_X_ATTRIBUTE, String.valueOf(this.max_vel_x)));
        configuration.addAttribute(new Attribute(MIN_IN_PLACE_VEL_THETA_ATTRIBUTE, String.valueOf(this.min_in_place_vel_theta)));
        configuration.addAttribute(new Attribute(MIN_VEL_THETA_ATTRIBUTE, String.valueOf(this.min_vel_theta)));
        configuration.addAttribute(new Attribute(MIN_VEL_X_ATTRIBUTE, String.valueOf(this.min_vel_x)));
        configuration.addAttribute(new Attribute(OCCDIST_SCALE_ATTRIBUTE, String.valueOf(this.occdist_scale)));
        configuration.addAttribute(new Attribute(OSCILLATION_RESET_DIST_ATTRIBUTE, String.valueOf(this.oscillation_reset_dist)));
        configuration.addAttribute(new Attribute(PDIST_SCALE_ATTRIBUTE, String.valueOf(this.pdist_scale)));
        configuration.addAttribute(new Attribute(RESTORE_DEFAULTS_ATTRIBUTE, String.valueOf(this.restore_defaults)));
        configuration.addAttribute(new Attribute(SIM_GRANULARITY_ATTRIBUTE, String.valueOf(this.sim_granularity)));
        configuration.addAttribute(new Attribute(SIM_TIME_ATTRIBUTE, String.valueOf(this.sim_time)));
        configuration.addAttribute(new Attribute(SIMPLE_ATTRACTOR_ATTRIBUTE, String.valueOf(this.simple_attractor)));
        configuration.addAttribute(new Attribute(VTHETA_SAMPLES_ATTRIBUTE, String.valueOf(this.vtheta_samples)));
        configuration.addAttribute(new Attribute(VX_SAMPLES_ATTRIBUTE, String.valueOf(this.vx_samples)));
        configuration.addAttribute(new Attribute(Y_VELS_ATTRIBUTE, String.valueOf(this.y_vels)));
        
        parent.appendChild(configuration);
    }        
}
