package de.unibi.airobots.btl.data.person;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.Arrays;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import de.unibi.airobots.btl.Type;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

/**
 * A list of ratings. Each rating indicates the possibility that two anchors in
 * different perceptual fields belong together.
 * 
 * @author dschulze
 */
public class RatingList extends Type {

	/**
	 * A concurrent hash map containing all ratings stored in this rating list.
	 * The ratings are indexed by a hash key generated out of 2 xcf id's.
	 */
	private Map<XcfHashKey, Double> values;

	/**
	 * To save computational resources, a rating map should not be updated every
	 * time a new rating is delivered. Instead an update should only be done if
	 * the new value differes with a certain amount from an old value, or if a
	 * completly new rating is inserted.
	 */
	private double tolerance;

	/**
	 * This flag indicates if the rating contains new rating values.
	 */
	private boolean updateNeeded;

	/** Constructor. */
	public RatingList() {
		values = new ConcurrentHashMap<XcfHashKey, Double>();
		updateNeeded = false;
	}

	/**
	 * Adds a new rating to this rating list.
	 * 
	 * @param key1
	 *            The xcf id of the first anchor.
	 * @param key2
	 *            The xcf id of the second anchor.
	 * @param value
	 *            The rating.
	 */
	public void addRating(String key1, String key2, double value) {

		XcfHashKey key = new XcfHashKey(key1, key2);
		addRating(key, value);

	}

	/**
	 * Adds a new rating to this rating list.
	 * 
	 * @param key
	 *            The xcf hash key builded from two different anchor ids.
	 * @param value
	 *            The rating.
	 */
	public void addRatingWOC(XcfHashKey key, double value) {
		values.put(key, value);
	}

	/**
	 * Adds a new rating to this rating list. Performes a check and updates the
	 * value only if it is a completly new value or if the difference to the old
	 * value is enough.
	 * 
	 * @param key
	 *            The xcf hash key builded from two different anchor ids.
	 * @param value
	 *            The rating.
	 */
	public void addRating(XcfHashKey key, double value) {

		if (values.containsKey(key)) {
			double oldValue = values.get(key);
			if (Math.abs(oldValue - value) > tolerance) {
				values.put(key, value);
				updateNeeded = true;
			}
		} else {
			values.put(key, value);
			updateNeeded = true;
		}

	}

	/**
	 * Returns the rating indexed by key1 and key2. If no rating is indexed by
	 * these values -1 is returned.
	 * 
	 * @param key1
	 *            The xcf id of the first anchor.
	 * @param key2
	 *            The xcf id of the second anchor.
	 * @return The rating or -1 if no rating exists.
	 */
	public double getRating(String key1, String key2) {

		XcfHashKey key = new XcfHashKey(key1, key2);
		if (values.containsKey(key)) {
			return values.get(key);
		}
		return -1;

	}

	/**
	 * Indicates if a new rating exceeded the tolerance value.
	 */
	public boolean shouldBeUpdated() {
		return updateNeeded;
	}

	/**
	 * Returns the number of ratings stored in this rating list.
	 * 
	 * @return The number of ratings.
	 */
	public int size() {
		return values.size();
	}

	public void setTolerance(float tolerance) {
		this.tolerance = tolerance;
	}

	public void setUpdateNeeded(boolean needed) {
		this.updateNeeded = needed;
	}

	public Map<XcfHashKey, Double> getRatings() {
		return values;
	}

	public void clearValues() {
		values.clear();
	}

	@Override
	public String toString() {

		StringBuilder builder = new StringBuilder();
		builder.append("Generator: ").append(getGenerator()).append("\n");
		builder.append("Ratings: \n");
		builder.append(values);

		return builder.toString();

	}

	/**
	 * Removes every rating associated with this id from the rating list.
	 */
	public void removeID(String id) {

		for (XcfHashKey key : values.keySet()) {
			if (key.getKey1().equals(id)) {
				values.remove(key);
				continue;
			}
			if (key.getKey2().equals(id)) {
				values.remove(key);
				continue;
			}
		}

	}

	/**
	 * Removes all childnodes with tag RATING from the element.
	 */
	public void sanitizeElement(Element parent) {

		// Remove timestamp and generator element
		// from parent element.
		Nodes nodes = parent.query("GENERATOR");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {
			parent.removeChild(nodes.get(i));
		}

		nodes = parent.query("TIMESTAMP");
		nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {
			parent.removeChild(nodes.get(i));
		}

		// Remove all RATING child nodes from parent
		// element.
		nodes = parent.query("RATING");
		nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {
			parent.removeChild(nodes.get(i));
		}

	}

	/** Returns the base tag of any rating list. */
	public static String getBaseTag() {
		return "RATINGS";
	}

	/** {@inheritDoc} */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// Insert values.
		for (Map.Entry<XcfHashKey, Double> entry : values.entrySet()) {
			XcfHashKey key = entry.getKey();
			double value = entry.getValue();

			Attribute attr1 = new Attribute("anchorId1", key.getKey1());
			Attribute attr2 = new Attribute("anchorId2", key.getKey2());
			Attribute attr3 = new Attribute("value", String.valueOf(value));

			Element element = new Element("RATING");
			element.addAttribute(attr1);
			element.addAttribute(attr2);
			element.addAttribute(attr3);

			parent.appendChild(element);

		}

	}

	/** {@inheritDoc} */
	public static void fromElement(Element element, RatingList list) {

		Type.fromElement(element, list);

		Nodes nodes = element.query("RATING");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element ratingElement = (Element) node;
				String key1 = ratingElement.getAttributeValue("anchorId1");
				String key2 = ratingElement.getAttributeValue("anchorId2");
				double value = Float.valueOf(ratingElement
						.getAttributeValue("value"));

				list.addRating(key1, key2, value);

			}

		}

	}

	/**
	 * This class is used to build a unique hash key out of two xcf ids
	 * (String).
	 */
	public class XcfHashKey {

		private String[] keys;

		/** Constructor. */
		XcfHashKey(String key1, String key2) {

			keys = new String[2];
			keys[0] = key1;
			keys[1] = key2;

			Arrays.sort(keys);

		}

		/** Returns the first key of the xcf hash key. */
		public String getKey1() {
			return keys[0];
		}

		/** Return the second key of the xcf hash key. */
		public String getKey2() {
			return keys[1];
		}

		@Override
		public boolean equals(Object obj) {

			if (this == obj) {
				return true;
			}

			if (obj == null) {
				return false;
			}

			if (this.getClass() != obj.getClass()) {
				return false;
			}

			XcfHashKey other = (XcfHashKey) obj;

			if (!Arrays.equals(keys, other.keys)) {
				return false;
			}

			return true;

		}

		@Override
		public int hashCode() {
			return Arrays.hashCode(keys);
		}

		@Override
		public String toString() {
			return Arrays.toString(keys);
		}

	}

}
