package de.unibi.airobots.btl.data.speechrec;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.Iterator;
import java.util.LinkedHashMap;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;

/**
 * Domain class representing an utterance. An utterance consisting of a word
 * sequence and a grammar tree. The grammar tree is null if the speech
 * recognizer does not configured to provide one. Currently the grammar tree is
 * unfortunately always null if the input is simulated.
 * 
 * @author lschilli
 */
public class Utterance extends Type implements Iterable<UtterancePart> {

	private static final String ATTR_STABLE = "stable";
	private static final String TAG_SCORE = "score";
	private static final String ATTR_LM = "lm";
	private static final String ATTR_COMBINED = "combined";
	private static final String ATTR_ACOUSTIC = "acoustic";
	private static final String ATTR_END = "end";
	private static final String ATTR_BEGIN = "begin";
	private static final String ATRR_ID = "id";
	private static final String TAG_PART = "part";
	private static final String XPATH_SPEECH_HYP_SEQ = "/speech_hyp/seq/*";
	private static final String ATTR_FAULT = "fault";
	private static final String ATTR_SKIP = "skip";
	private static final String ATTR_CANCEL = "cancel";
	private static final String ATTR_REFID = "refid";
	private static final String TAG_GRAMMARTREE = "grammartree";
	private static final String ATTRIB_NAME = "name";
	private static final String TAG_NONTERMINAL = "nonterminal";
	private static final String TAG_TERMINAL = "terminal";

	private LinkedHashMap<Integer, UtterancePart> utterancePartMap = new LinkedHashMap<Integer, UtterancePart>();
	private boolean stable;
	private GrammarTree grammarTree;

	public Utterance() {
		setLegacyParsing(true);
	}

	public static String getBaseTag() {
		return "speech_hyp";
	}

	public static void fromElement(Element element, Utterance type) {

		Type.fromElement(element, type);

		type.setStable(ElementParser.getBooleanAttributeValueUnchecked(element,
				ATTR_STABLE));
		parseSeq(element, type);
		type.setGrammarTree(parseGrammarTree(element, type));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		// TODO bad hack
		throw new RuntimeException("Operation not supported atm.");

	}

	public GrammarTree getGrammarTree() {
		return grammarTree;
	}

	void setGrammarTree(GrammarTree grammarTree) {
		this.grammarTree = grammarTree;
	}

	public boolean isStable() {
		return stable;
	}

	private void setStable(boolean stable) {
		this.stable = stable;
	}

	private void addUtterancePart(UtterancePart uttp) {
		utterancePartMap.put(uttp.getId(), uttp);
	}

	public UtterancePart getUtterancePartById(int refid) {
		return utterancePartMap.get(refid);
	}

	public Iterator<UtterancePart> iterator() {
		return utterancePartMap.values().iterator();
	}

	public int getUtterancePartCount() {
		return utterancePartMap.size();
	}

	private static void handleSymbolSequence(Element elem, Utterance utt,
			GrammarNonTerminal supterm) {

		Elements elems = elem.getChildElements();
		for (int i = 0; i < elems.size(); i++) {
			Element e = elems.get(i);
			if (TAG_TERMINAL.equals(e.getLocalName())) {
				UtterancePart uttp = utt.getUtterancePartById(ElementParser
						.getIntAttributeValue(e, ATTR_REFID));
				if (uttp == null) {
					throw new RuntimeException(
							"Cannot resolve word id in grammartree");
				}
				supterm.addSymbol(uttp);
			} else if (TAG_NONTERMINAL.equals(e.getLocalName())) {
				GrammarNonTerminal nonterm = new GrammarNonTerminal();
				nonterm.setName(ElementParser.getAttributeValue(e, ATTRIB_NAME));
				handleSymbolSequence(e, utt, nonterm);
				supterm.addSymbol(nonterm);
			}
		}

	}

	private static GrammarTree parseGrammarTree(Element root, Utterance utt) {

		GrammarTree tree = new GrammarTree();
		Element grmelem = root.getFirstChildElement(TAG_GRAMMARTREE);
		if (grmelem != null) {
			// handles empty case (silence) (hotfix)
			if (grmelem.getAttribute(ATTR_CANCEL) == null) {
				return null;
			}
			tree.setCancel(ElementParser.getBooleanAttributeValue(grmelem,
					ATTR_CANCEL));
			tree.setSkip(ElementParser.getBooleanAttributeValue(grmelem,
					ATTR_SKIP));
			tree.setFault(ElementParser.getBooleanAttributeValue(grmelem,
					ATTR_FAULT));
			handleSymbolSequence(grmelem, utt, tree);
			return tree;
		} else {
			return null;
		}

	}

	private static void parseSeq(Element root, Utterance utt) {

		Nodes nodes = root.query(XPATH_SPEECH_HYP_SEQ);
		UtterancePart uttp = null;
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (TAG_PART.equals(elem.getLocalName())) {
					uttp = new UtterancePart();
					uttp.setWord(ElementParser.getTagValue(elem));
					uttp.setId(ElementParser
							.getIntAttributeValue(elem, ATRR_ID));
					uttp.setBegin(ElementParser.getIntAttributeValue(elem,
							ATTR_BEGIN));
					uttp.setEnd(ElementParser.getIntAttributeValue(elem,
							ATTR_END));
				} else if (TAG_SCORE.equals(elem.getLocalName())) {
					uttp.setAcousticScore(ElementParser
							.getDoubleAttributeValue(elem, ATTR_ACOUSTIC));
					uttp.setCombinedScore(ElementParser
							.getDoubleAttributeValue(elem, ATTR_COMBINED));
					uttp.setLmScore(ElementParser.getDoubleAttributeValue(elem,
							ATTR_LM));
					utt.addUtterancePart(uttp);
				}

			}
		}

	}

	public String getSimpleString() {

		StringBuilder strb = new StringBuilder();
		for (UtterancePart uttp : utterancePartMap.values()) {
			if (strb.length() != 0)
				strb.append(" ");
			strb.append(uttp.getWord());
		}

		return strb.toString();

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {

		StringBuilder strb = new StringBuilder();
		strb.append("(" + getTimestamp() + ")");
		for (UtterancePart uttp : utterancePartMap.values()) {
			strb.append(" ");
			strb.append(uttp.toString());
		}
		strb.append(" ;");

		return strb.toString();

	}

}
