package de.unibi.airobots.btl.data.speechrec;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;

/**
 * Domain class representing a word in the speech recognizers output. Each part
 * is associated with an acoustic, a language model and a combined score.
 * 
 * @author lschilli
 * @author sjebbara
 */
public class UtterancePart extends Type implements GrammarSymbol {

	private String word;
	private int id;
	private int begin;
	private int end;
	private double acousticScore;
	private double lmScore;
	private double combinedScore;

	public String getWord() {
		return word;
	}

	public void setWord(String word) {
		this.word = word;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public int getBegin() {
		return begin;
	}

	public void setBegin(int begin) {
		this.begin = begin;
	}

	public int getEnd() {
		return end;
	}

	public void setEnd(int end) {
		this.end = end;
	}

	public double getAcousticScore() {
		return acousticScore;
	}

	public void setAcousticScore(double acousticScore) {
		this.acousticScore = acousticScore;
	}

	public double getLmScore() {
		return lmScore;
	}

	public void setLmScore(double lmScore) {
		this.lmScore = lmScore;
	}

	public double getCombinedScore() {
		return combinedScore;
	}

	public void setCombinedScore(double combinedScore) {
		this.combinedScore = combinedScore;
	}

	public static String getBaseTag() {
		return "UTTERANCEPART";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(acousticScore);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + begin;
		temp = Double.doubleToLongBits(combinedScore);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + end;
		result = prime * result + id;
		temp = Double.doubleToLongBits(lmScore);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + ((word == null) ? 0 : word.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (getClass() != obj.getClass())
			return false;
		UtterancePart other = (UtterancePart) obj;
		if (Double.doubleToLongBits(acousticScore) != Double
				.doubleToLongBits(other.acousticScore))
			return false;
		if (begin != other.begin)
			return false;
		if (Double.doubleToLongBits(combinedScore) != Double
				.doubleToLongBits(other.combinedScore))
			return false;
		if (end != other.end)
			return false;
		if (id != other.id)
			return false;
		if (Double.doubleToLongBits(lmScore) != Double
				.doubleToLongBits(other.lmScore))
			return false;
		if (word == null) {
			if (other.word != null)
				return false;
		} else if (!word.equals(other.word))
			return false;
		return true;
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);
		Element actionElement = new Element("UtterancePartData");
		actionElement.addAttribute(new Attribute("word", word));
		actionElement.addAttribute(new Attribute("id", String.valueOf(id)));
		actionElement
				.addAttribute(new Attribute("begin", String.valueOf(begin)));
		actionElement.addAttribute(new Attribute("end", String.valueOf(end)));
		actionElement.addAttribute(new Attribute("acousticScore", String
				.valueOf(acousticScore)));
		actionElement.addAttribute(new Attribute("lmScore", String
				.valueOf(lmScore)));
		actionElement.addAttribute(new Attribute("combinedScore", String
				.valueOf(combinedScore)));
		parent.appendChild(actionElement);
	}

	/** @{inheritDoc */
	public static void fromElement(Element element, UtterancePart type) {

		Type.fromElement(element, type);

		Nodes positionNodes = element.query("UtterancePartData");
		Element positionElement = (Element) positionNodes.get(0);

		type.setWord(ElementParser.getAttributeValue(positionElement, "word"));
		type.setId(ElementParser.getIntAttributeValue(positionElement, "id"));
		type.setBegin(ElementParser.getIntAttributeValue(positionElement,
				"begin"));
		type.setEnd(ElementParser.getIntAttributeValue(positionElement, "end"));
		type.setAcousticScore(ElementParser.getDoubleAttributeValue(
				positionElement, "acousticScore"));
		type.setLmScore(ElementParser.getDoubleAttributeValue(positionElement,
				"lmScore"));
		type.setCombinedScore(ElementParser.getDoubleAttributeValue(
				positionElement, "combinedScore"));
	}

	@Override
	public String toString() {
		return "UtterancePart [word=" + word + "]";
	}

}
